from escher.version import (
    __version__,
    __schema_version__,
    __map_model_version__,
)
import os
import re
from os.path import dirname, realpath, join

root_directory = realpath(join(dirname(__file__), '..'))

# relative to root_directory
_escher_local = {
    'map_jsonschema': 'escher/static/jsonschema/1-0-0',
}

_escher_web = {
    'server_index': '%s/%s/index.json' % (__schema_version__,
                                          __map_model_version__),
    'map_download': '%s/%s/maps/' % (__schema_version__,
                                     __map_model_version__),
    'model_download': '%s/%s/models/' % (__schema_version__,
                                         __map_model_version__),
}

_dependencies_cdn = {
    'escher': 'https://unpkg.com/escher@%s/dist/escher.js' % __version__,
    'escher_min': ('https://unpkg.com/escher@%s/dist/escher.min.js' %
                   __version__),
}

_links = {
    'escher_root': 'https://escher.github.io/',
    'github': 'https://github.com/zakandrewking/escher',
    'github_releases': 'https://github.com/zakandrewking/escher/releases',
    'documentation': 'https://escher.readthedocs.org/',
}


def get_filepath(key):
    """Get the filepath for the key"""
    if key in _escher_local:
        return join(root_directory, _escher_local[key])
    else:
        raise Exception('File key not recognized: %s' % key)


def get_url(name):
    """Get a url for the key"""

    if name in _escher_web:
        return _links['escher_root'] + _escher_web[name]
    elif name in _links:
        return _links[name]
    elif name in _dependencies_cdn:
        return _dependencies_cdn[name]
    else:
        raise Exception('name not found')



# global config
rc = {}


from escher.version import (
    __version__,
    __schema_version__,
    __map_model_version__,
)

from escher.plots import (
    Builder,
    list_available_maps,
    list_available_models,
)


def _jupyter_nbextension_paths():
    return [{
        'section': 'notebook',
        'src': 'static',
        'dest': 'escher',
        'require': 'escher/extension'
    }]



import base64
import json


def b64dump(data):
    """Returns the base64 encoded dump of the input

    Arguments
    ---------

    data: Can be a dict, a (JSON or plain) string, or None

    """
    if isinstance(data, dict):
        data = json.dumps(data)
    elif data is None:
        data = json.dumps(None)
    return base64.b64encode(data.encode('utf-8')).decode('utf-8')



import json
from collections import defaultdict
import os
from os.path import join, dirname
on_rtd = os.environ.get('READTHEDOCS', None) == 'True'

def get_full_version(main_version, post_version=None):
    """Generate a PEP440 compliant version with an optional post-release."""
    if post_version is None:
        return main_version
    else:
        return '%s.post%s' % (main_version, post_version)

if on_rtd:
    # ReadTheDocs does not have a full environment, so fill in some empty values
    # to get autodoc going
    package = defaultdict(str)
else:
    with open(join(dirname(__file__), 'static', 'package.json'), 'r') as f:
        package = json.load(f)

# software version
__version__ = package['version']
# Escher schema and map version. The schema version determines the version of
# the map specification, and the map version is an additional level so that maps
# can be released with the same spec but new layouts.
__schema_version__ = package['schema_version']
__map_model_version__ = package['map_model_version']

# post-release version (required by PyPI & PEP440)
__post_version__ = package['post_version']
# full version with post-release
__full_version__ = get_full_version(__version__, __post_version__)



import sys
from os.path import join, dirname, realpath

from setuptools import setup, find_packages

directory = dirname(realpath(__file__))
sys.path.insert(0, join(directory, 'escher'))
version = __import__('version').__version__
full_version = __import__('version').__full_version__
package = __import__('version').package
port = 8789

setup(
    name='Escher',
    version=full_version,
    author=package['author'],
    url=package['homepage'],
    description=package['description'],
    keywords=', '.join(package['keywords']),
    license=package['license'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: MIT License',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Visualization',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Operating System :: OS Independent'
    ],
    packages=find_packages(),
    include_package_data=True,
    data_files=[
        (
            'share/jupyter/nbextensions/escher',
            [
                'escher/static/extension.js',
                'escher/static/escher.min.js',
                'escher/static/escher.min.js.map',
            ]
        ),
        (
            'etc/jupyter/nbconfig/notebook.d',
            ['escher.json'],
        )
    ],
    install_requires=[
        'Jinja2>=2.7.3,<3',
        'pytest>=4.0.1,<5',
        'cobra>=0.5.0',
        'jsonschema>=3.0.1,<4',
        'ipywidgets>=7.4.0,<8',
        'pandas>=0.18'
    ],
    extras_require={
        'docs': [
            'sphinx>=2.1.1,<3',
            'sphinx-rtd-theme>=0.4.3,<0.5',
            'nbsphinx>=0.4.2,<0.5'
        ],
    },
)
